.TH SqlGawk Onyx
.SH NAME
SqlGawk.ppg \- Base-Grammar for SQL's
.SH SYNOPSYIS
.nf
class SqlGawk : public SqlPipe {
	HashList *sql_table_hash;
	HashList *sql_field_hash;
	
	Text *sql_tables;
	Text *sql_fields;
	
	char *sql_condition;
	char *sql_action;
	char *sql_expression;
	
	char *sql_path;
	
	int  distinct_flag;
	
	SqlGawk *sql_subselect;
	
	SqlGawk();
	virtual ~SqlGawk( );
	
	
	virtual int	sql();
	virtual int	statement();
	
	virtual int	value();
	virtual int	compop();
	virtual int	boolop();
	virtual int	numop();
	virtual int	expression();
	virtual int	condition();
	virtual int	where();
	
	virtual int	begin();
	virtual int	commit();
	virtual int	select();
	virtual int	insert();
	virtual int	del();
	virtual int	update();
	
	
	virtual int	sql_gawk(char *cmd,char *file);
	virtual int 	sql_open();
	virtual void	sql_purge();
	virtual char	*sql_lookup_table(char *tname);
	virtual char	*sql_lookup_field(char *fname);
	}
.fi
.SH DESCRIPTION
SqlGawk is GNU-AWK as a SQL-Engine. If Y think this is stupid,
think about accessing any plain tables text by sql.

Also this engine can be usefull for smaller databases. Six
selects per second is fast enough for tables smaler than 200 rows.
And there is no need for a real engine, if Y like do distribute
a demo version of Yr application.

SqlGawk has some disadvantages. Its only N:0 relational, so
it dont understand joins. And it also dont understand aggregates.

Its posible to solve this by implementing all whats in SHQL
from Bruce Momjian. SqlGawk is compatible to SHQL.
.SH IMPLEMENTATION
.PP
sql : 
.br
.in +0.8i
.ad l
.nh
statement  ";" | true_  
.br
;
.br
.hy

A Statement has to end with ";" or a message "error:illegal statement"
is raised.
.PP
statement : 
.br
.in +0.8i
.ad l
.nh
database  | begin  | commit  | select  | insert  | del  | update  
.br
;
.br
.hy

Only a small subset of SQL is valid here.
.PP
value : 
.br
.in +0.8i
.ad l
.nh
"\(rs"" match  "\(rs"" | num  | "not" expression  | "(" expression  ")" | id  
.br
;
.br
.hy

A value can be a string, a number, a not expression, a
expression in braces, or a identifier.
.PP
compop : 
.br
.in +0.8i
.ad l
.nh
"=" | "!=" | "<>" | "<=" | ">=" | "<" | ">" | "like" 
.br
;
.br
.hy

Here the different compare opperators are translated.
Eg. "like" is translated to "~", and "=" is translated
to "==".
.PP
boolop : 
.br
.in +0.8i
.ad l
.nh
"and" | "or" 
.br
;
.br
.hy

Here the boolean operators are translated.
.PP
numop : 
.br
.in +0.8i
.ad l
.nh
"+" | "-" | "*" | "/" | "%" | "mod" 
.br
;
.br
.hy

Here the numeric operators are translated.
.PP
expression : 
.br
.in +0.8i
.ad l
.nh
value  ( compop  value  | boolop  value  | numop  value  ) 
.br
;
.br
.hy

A expression starts with a value and perhaps continues with
operator value pairs.
.PP
condition : 
.br
.in +0.8i
.ad l
.nh
expression  
.br
;
.br
.hy

A condition is nothing else than a expression. This is done
because PPG is only LL(1) and I dont have any better solution
in mind.

Caution no type checking is done, perhaps in a later version.
.PP
where : 
.br
.in +0.8i
.ad l
.nh
"where" condition  
.br
;
.br
.hy

The "where" statement only has a condition.
.PP
begin : 
.br
.in +0.8i
.ad l
.nh
"begin" matchexp  
.br
;
.br
.hy

Caution : This is realy doing nothing !
.PP
commit : 
.br
.in +0.8i
.ad l
.nh
"commit" matchexp  
.br
;
.br
.hy

Caution : This is realy doing nothing !
.PP
select : 
.br
.in +0.8i
.ad l
.nh
"select" [ "distinct" ] ( id  | "*" ! "," ) "from" id  [ where  ] 
.br
;
.br
.hy

This is doing the job of translating a select statement to awk.
.PP
insert : 
.br
.in +0.8i
.ad l
.nh
"insert" "into" id  [ "(" ( id  ! "," ) ")" ] "values" "(" ( num  | "\(rs"" match  "\(rs"" ! "," ) ")" 
.br
;
.br
.hy

This is doing the job of translating a insert statement to awk.
.PP
del : 
.br
.in +0.8i
.ad l
.nh
"delete" "from" id  [ where  ] 
.br
;
.br
.hy

This is doing the job of translating a delete statement to awk.
.PP
update : 
.br
.in +0.8i
.ad l
.nh
"update" id  "set" ( id  "=" expression  ! "," ) [ where  ] 
.br
;
.br
.hy

This is doing the job of translating a update statement to awk.
.SH AUTHOR
Michael Koehne <kraehe@bakunin.north.de>
