.TH SqlIngres Onyx
.SH NAME
SqlIngres.ppg \- Grammar for SQL to Ingres
.SH SYNOPSYIS
.nf
class SqlIngres : public SqlParser {
	HashList *sql_table_hash;
	HashList *sql_field_hash;
	
	Text *sql_tables;
	Text *sql_fields;
	
	char *quel_cmd;
	char range[64];
	int like_flag;
	
	SqlIngres();
	virtual ~SqlIngres();
	void GetInfo();
	
	virtual int sql_open();
	virtual int sql_close();
	
	virtual int sql();
	virtual int statement();
	
	virtual int colid();
	virtual int value();
	virtual int compop();
	virtual int boolop();
	virtual int numop();
	virtual int expression();
	virtual int condition();
	
	virtual int where();
	virtual int del();
	virtual int update();
	virtual int insert();
	virtual int select();
	virtual int type();
	virtual int create();
	virtual int drop();
	virtual int load();
	virtual int unload();
	virtual int grant();
	virtual int revoke();
	
	virtual int to_din(char *str);
	virtual int from_din(char *str);
	}
.fi
.SH DESCRIPTION
SqlIngres is Ingres89 as a SQL-Engine. It makes the necessary
translations from SQL to QUEL, and calls EQUEL-libq directly.

I dont know if this is portable with ASK/Ingres ?

SqlIngres has some disadwantages related to this stupid parser.

SqlIngres doesnt have aggegate functions now, I think I'll
implement it in one of next versions. If Y need it ask me
for my support and hack it.

SqlIngres mostly works like SqlGawk.
.SH IMPLEMENTATION
.PP
sql : 
.br
.in +0.8i
.ad l
.nh
statement  ";" | true  
.br
;
.br
.hy

A Statement has to end with ";" or a message "error:illegal statement"
is raised.
.PP
statement : 
.br
.in +0.8i
.ad l
.nh
database  | del  | update  | insert  | select  | create  | drop  | load  | unload  | grant  | revoke  | matchexp  
.br
;
.br
.hy

Only a small subset of SQL is valid here.
.PP
colid : 
.br
.in +0.8i
.ad l
.nh
id  [ "." id  ] 
.br
;
.br
.hy

This matches a colum identifier.
.PP
value : 
.br
.in +0.8i
.ad l
.nh
"\(rs"" match  "\(rs"" | num  | "not" expression  | "(" expression  ")" | colid  
.br
;
.br
.hy

A value can be a string, a number, a not expression, a expression
in braces or, a identifier. Identifiers are translated here betwen
SQL and QUEL acording previous range statemenst.
.PP
compop : 
.br
.in +0.8i
.ad l
.nh
"=" | "!=" | "<>" | "<=" | ">=" | "<" | ">" | "like" 
.br
;
.br
.hy

Here the different compare opperators are translated.

A like operator is translated to "=" and a flag is raised
for later be used in string values.

So its posible to write 'colid like "string"' but not
'colid like colid'
.PP
boolop : 
.br
.in +0.8i
.ad l
.nh
"and" | "or" 
.br
;
.br
.hy

Here the boolean operators are translated.
.PP
numop : 
.br
.in +0.8i
.ad l
.nh
"+" | "-" | "*" | "/" 
.br
;
.br
.hy

Here the numeric operators are translated.
.PP
expression : 
.br
.in +0.8i
.ad l
.nh
value  ( compop  value  | boolop  value  | numop  value  ) 
.br
;
.br
.hy

A expression starts with a value and perhaps continues with
operator value pairs.
.PP
condition : 
.br
.in +0.8i
.ad l
.nh
expression  
.br
;
.br
.hy

A condition is nothing else than a expression. This is done
because PPG is only LL(1) and I dont have any better solution
in mind.

Caution no type checking is done, perhaps in a later version.
.PP
where : 
.br
.in +0.8i
.ad l
.nh
[ "where" condition  ] 
.br
;
.br
.hy

The "where" statement only has a condition.
.PP
del : 
.br
.in +0.8i
.ad l
.nh
"delete" "from" id  where  
.br
;
.br
.hy

This is doing the job of translating a delete statment to
a range and a quel-delete statement and
calling IIwrite and IIsync for doing the job.
.PP
update : 
.br
.in +0.8i
.ad l
.nh
"update" id  "set" ( colid  "=" expression  ! "," ) where  
.br
;
.br
.hy

This is doing the job of translating a update statment to
a range and a quel-replace statement and
calling IIwrite and IIsync for doing the job.
.PP
insert : 
.br
.in +0.8i
.ad l
.nh
"insert" "into" id  [ "(" ( colid  ! "," ) ")" | true  ] "values" "(" ( num  | "\(rs"" match  "\(rs"" ! "," ) ")" 
.br
;
.br
.hy

This is doing the job of translating a insert statment to
a range and a quel-append statement and
calling IIwrite and IIsync for doing the job.
.PP
select : 
.br
.in +0.8i
.ad l
.nh
"select" ( colid  | "*" ! "," ) "from" ( id  ! "," ) where  
.br
;
.br
.hy

This is doing the job of translating a select statment to
a range and a quel-retrieve statement and
calling IIwrite, IIsync, IIsetup, IIn_get, IIn_ret
for doing the job.

Important note : Y need a patched IIn_ret to run this !
.PP
type : 
.br
.in +0.8i
.ad l
.nh
"integer" | "smallint" | "serial" | "smallfloat" | "real" | "float" | "date" | "double" "precision" | "char" [ "(" num  ")" ] | "varchar" "(" num  ")" | "decimal" [ "(" num  [ "," num  ] ")" ] | "dec" [ "(" num  [ "," num  ] ")" ] | "numeric" [ "(" num  [ "," num  ] ")" ] | "money" [ "(" num  [ "," num  ] ")" ] 
.br
;
.br
.hy
.PP
create : 
.br
.in +0.8i
.ad l
.nh
"create" "table" id  "(" ( "constraint" id  "check" "(" match  ")" | "primary" "key" "(" match  ")" | "foreign" "key" "(" match  ")" "references" id  [ "on" [ "delete" | "update" ] [ "cascade" | "restricted" | "set" id  ] ] | colid  type  ! "," ) ")" matchexp  | "index" id  "on" id  "(" id  ")" 
.br
;
.br
.hy
.PP
drop : 
.br
.in +0.8i
.ad l
.nh
"drop" "table" id  | "index" id  
.br
;
.br
.hy
.PP
load : 
.br
.in +0.8i
.ad l
.nh
"load" "from" "\(rs"" match  "\(rs"" "insert" "into" id  [ "commit" num  ] 
.br
;
.br
.hy
.PP
unload : 
.br
.in +0.8i
.ad l
.nh
"unload" "to" "\(rs"" match  "\(rs"" select  
.br
;
.br
.hy
.PP
grant : 
.br
.in +0.8i
.ad l
.nh
"grant" "all" "on" id  "to" id  
.br
;
.br
.hy
.PP
revoke : 
.br
.in +0.8i
.ad l
.nh
"revoke" "all" "on" id  "from" id  
.br
;
.br
.hy
.SH AUTHOR
Michael Koehne <kraehe@bakunin.north.de>
